@@ -5,6 +5,7 @@ import pkg_resources
 import re
 import string
 import textwrap
+from webob import Response
 
 from xblock.core import XBlock, XML_NAMESPACES
 from xblock.fields import Scope, Float, String, Dict, List, Integer, Boolean
@@ -105,6 +106,14 @@ class HastexoXBlock(XBlock,
         help="Timeout for how long to wait before suspending a stack, after "
              "the last keepalive was received from the browser, in seconds. "
              "Takes precedence over the globally defined timeout.")
+    stack_key_type = String(
+        values=["rsa", "ed25519", ""],
+        default="",
+        scope=Scope.settings,
+        help="Key type for generating an SSH key for accessing the lab. "
+             "If set to None, the key handling should be done via the lab "
+             "template."
+    )
     hook_script = String(
         scope=Scope.settings,
         help="The relative path to an uploaded executable script. "
@@ -153,6 +162,13 @@ class HastexoXBlock(XBlock,
              "not display hints and feedback. Default is \"Progress check "
              "result\"."
     )
+    enable_fullscreen = String(
+        values=["true", "false", "inherit"],
+        default="inherit",
+        scope=Scope.settings,
+        help="Enable the learners to launch a lab in fullscreen mode on a "
+             "separate browser window. Overrides the globally defined setting."
+    )
 
     # Set via XML
     hook_events = Dict(
@@ -225,6 +241,7 @@ class HastexoXBlock(XBlock,
         'hook_events',
         'stack_user_name',
         'stack_protocol',
+        'stack_key_type',
         'launch_timeout',
         'suspend_timeout',
         'delete_age',
@@ -232,7 +249,8 @@ class HastexoXBlock(XBlock,
         'providers',
         'tests',
         'read_only',
-        'hidden')
+        'hidden',
+        'enable_fullscreen')
 
     has_author_view = True
     has_score = True
@@ -441,11 +459,14 @@ class HastexoXBlock(XBlock,
         node.set('stack_user_name', self.stack_user_name)
         node.set('stack_protocol', self.stack_protocol)
         node.set('stack_template_path', self.stack_template_path or '')
+        node.set('stack_key_type', self.stack_key_type)
         node.set('launch_timeout', str(self.launch_timeout or ''))
         node.set('suspend_timeout', str(self.suspend_timeout or ''))
         node.set('hook_script', self.hook_script or '')
         node.set('delete_age', str(self.delete_age or ''))
         node.set('read_only', str(self.read_only))
+        node.set('hidden', str(self.hidden))
+        node.set('enable_fullscreen', self.enable_fullscreen)
 
         # Include nested blocks in course export
         if self.has_children:
@@ -531,6 +552,17 @@ class HastexoXBlock(XBlock,
             # delete_age value in settings is in days, convert to seconds
             return settings.get("delete_age", 14) * 86400
 
+    def get_enable_fullscreen(self, settings):
+        """
+        Return 'enable_fullscreen' mode value. XBlock attribute
+        overrides the global setting, if not set to inherit the global setting.
+        """
+        enable_fullscreen = settings.get("enable_fullscreen", False)
+        if self.enable_fullscreen != "inherit":
+            enable_fullscreen = bool(strtobool(self.enable_fullscreen))
+
+        return enable_fullscreen
+
     def get_suspend_timeout(self):
         """
         Return 'suspend_timeout' in seconds.
@@ -555,20 +587,96 @@ class HastexoXBlock(XBlock,
 
         return stack_name
 
-    @staticmethod
-    def _get_text_js_url():
+    def get_js_urls(self):
         """
-        Returns the Javascript translation file
-        for the currently selected language, if supported.
+        Returns a dict of urls for the Javascript files.
         """
+        settings = get_xblock_settings()
+        plugins_url = self.runtime.local_resource_url(
+            self, 'public/js/plugins.js')
+
+        # guacamole common library url
+        guac_js_version = settings.get("guacamole_js_version", "1.5.4")
+        guac_common_url = (
+            self.runtime.local_resource_url(
+                self,
+                f'public/js/guacamole-common-js/{guac_js_version}-all.min.js'))
+
+        # HastexoGuacamoleClient url
+        guac_client_url = self.runtime.local_resource_url(
+            self, 'public/js/client.js')
+
+        # HastexoXBlock url
+        main_js_url = self.runtime.local_resource_url(
+            self, 'public/js/main.js')
+
+        js_urls = {
+            "plugins_url": plugins_url,
+            "guac_common_url": guac_common_url,
+            "guac_client_url": guac_client_url,
+            "main_js_url": main_js_url,
+        }
+
+        # Translation file url (if supported)
         lang_code = translation.get_language()
         if lang_code and lang_code in SUPPORTED_LANGUAGES:
-            text_js = f'public/js/translations/{lang_code}/text.js'
-            if pkg_resources.resource_exists(loader.module_name, text_js):
-                return text_js
-        logger.warning(
-            "Javascript translation file missing or language is not supported")
-        return None
+            text_js_url = f'public/js/translations/{lang_code}/text.js'
+            if pkg_resources.resource_exists(loader.module_name, text_js_url):
+                js_urls["text_js_url"] = self.runtime.local_resource_url(
+                    self, text_js_url)
+        else:
+            logger.warning("Javascript translation file missing or "
+                           "language is not supported")
+        return js_urls
+
+    def get_context(self, stack=None):
+        settings = get_xblock_settings()
+        stack = stack or self.get_stack()
+
+        context = {
+            "stack_name": stack.name,
+            "terminal_url": settings.get("terminal_url"),
+            "keepalive_url": self.runtime.handler_url(self, 'keepalive'),
+            "timeouts": settings.get("js_timeouts"),
+            "has_tests": len(self.tests) > 0,
+            "protocol": self.stack_protocol,
+            "ports": self.ports,
+            "port": stack.port,
+            "instructions_layout": settings.get("instructions_layout"),
+            "read_only": self.read_only or self.hidden,
+            "hidden": self.hidden,
+            "progress_check_label": self.progress_check_label,
+            "show_hints_on_error": self.show_hints_on_error,
+            "show_feedback": self.show_feedback,
+            "progress_check_result_heading": self.progress_check_result_heading
+        }
+
+        return context
+
+    @XBlock.handler
+    def launch_new_window(self, request, suffix=''):
+        """
+        The fullscreen lab view, opened in a new browser window.
+        """
+        if 'sessionid' not in request.cookies:
+            return Response(status=401, body="Unauthorized")
+
+        # Get context
+        context = self.get_context()
+
+        # Pass the token (for keepalives)
+        context["csrftoken"] = request.cookies.get('csrftoken')
+
+        # Get the JavaScript urls
+        js_urls = self.get_js_urls().items()
+        for key, value in js_urls:
+            context[key] = value
+
+        # Render the lab template
+        template = loader.render_django_template(
+            "static/html/lab.html", context)
+
+        return Response(template)
 
     def student_view(self, context=None):
         """
@@ -584,6 +692,8 @@ class HastexoXBlock(XBlock,
         self.stack_run = "%s_%s" % (course_id.course, course_id.run)
         self.stack_name = self.get_stack_name()
 
+        enable_fullscreen = self.get_enable_fullscreen(settings)
+
         frag = Fragment()
 
         # Render children
@@ -598,51 +708,28 @@ class HastexoXBlock(XBlock,
         i18n_service = self.runtime.service(self, "i18n")
 
         frag.add_content(loader.render_django_template(
-            "static/html/main.html", {"child_content": child_content},
+            "static/html/main.html",
+            {"child_content": child_content,
+             "enable_fullscreen": enable_fullscreen},
             i18n_service=i18n_service
         ))
 
-        # Add the public CSS and JS
+        # Add the public CSS and JS files
         frag.add_css_url(
             self.runtime.local_resource_url(self, 'public/css/main.css')
         )
-        frag.add_javascript_url(
-            self.runtime.local_resource_url(self, 'public/js/plugins.js')
-        )
-        frag.add_javascript_url(
-            self.runtime.local_resource_url(self, 'public/js/main.js')
-        )
-        text_js_url = self._get_text_js_url()
-        if text_js_url:
-            frag.add_javascript_url(
-                self.runtime.local_resource_url(self, text_js_url)
-            )
-        guac_js_version = settings.get("guacamole_js_version", "1.4.0")
-        frag.add_javascript_url(
-            self.runtime.local_resource_url(
-                self,
-                f'public/js/guacamole-common-js/{guac_js_version}-all.min.js')
-        )
+        js_urls = list(self.get_js_urls().values())
+        for url in js_urls:
+            frag.add_javascript_url(url)
 
         # Create the stack in the database
         stack = self.create_stack(settings, course_id, student_id)
 
+        # Get context
+        context = self.get_context(stack)
+
         # Call the JS initialization function
-        frag.initialize_js('HastexoXBlock', {
-            "terminal_url": settings.get("terminal_url"),
-            "timeouts": settings.get("js_timeouts"),
-            "has_tests": len(self.tests) > 0,
-            "protocol": self.stack_protocol,
-            "ports": self.ports,
-            "port": stack.port,
-            "instructions_layout": settings.get("instructions_layout"),
-            "read_only": self.read_only or self.hidden,
-            "hidden": self.hidden,
-            "progress_check_label": self.progress_check_label,
-            "show_hints_on_error": self.show_hints_on_error,
-            "show_feedback": self.show_feedback,
-            "progress_check_result_heading": self.progress_check_result_heading
-        })
+        frag.initialize_js('HastexoXBlock', context)
 
         return frag
 
@@ -798,7 +885,8 @@ class HastexoXBlock(XBlock,
             result = self.launch_stack_task(settings, {
                 "stack_id": stack.id,
                 "reset": reset,
-                "learner_id": stack.learner.id
+                "learner_id": stack.learner.id,
+                "stack_key_type": self.stack_key_type
             })
 
             # Update stack
