@@ -203,6 +203,7 @@ class LaunchStackTask(HastexoTask):
         self.stack_run = stack.run
         self.stack_name = stack.name
         self.stack_user_name = stack.user
+        self.stack_key = stack.key
         self.course_id = stack.course_id
         self.student_id = stack.student_id
         self.hook_events = stack.hook_events
@@ -254,12 +255,6 @@ class LaunchStackTask(HastexoTask):
         except LaunchStackFailed as e:
             logger.error(e.error_msg)
 
-            # In case of failure, only return the provider if this was a failed
-            # resume attempt.
-            provider_name = ""
-            if e.suspend:
-                provider_name = e.provider.name
-
             stack_data = {
                 'status': e.status,
                 'error_msg': textwrap.shorten(e.error_msg, width=256),
@@ -267,9 +262,15 @@ class LaunchStackTask(HastexoTask):
                 'user': "",
                 'key': "",
                 'password': "",
-                'provider': provider_name
+                'provider': ""
             }
 
+            # In case of a failed resume attempt, return the provider
+            # and keep the key in place so that the learner can retry.
+            if e.suspend:
+                stack_data['provider'] = e.provider.name
+                stack_data.pop('key')
+
             # Roll back in case of failure
             self.cleanup_stack(e)
 
@@ -457,8 +458,11 @@ class LaunchStackTask(HastexoTask):
                     self.sleep()
 
                     logger.info("Creating stack [%s]." % self.stack_name)
-                    provider_stack = provider.create_stack(self.stack_name,
-                                                           self.stack_run)
+                    provider_stack = provider.create_stack(
+                        self.stack_name,
+                        self.stack_run,
+                        key_type=self.stack_key_type)
+
             except ProviderException as e:
                 error_msg = ("Error creating stack [%s]: %s" %
                              (self.stack_name, e))
@@ -494,7 +498,7 @@ class LaunchStackTask(HastexoTask):
             # Launch completed successfully.  Wait for provisioning, collect
             # its IP address, and save the private key.
             try:
-                check_data = self.check_stack(provider_stack["outputs"],
+                check_data = self.check_stack(provider_stack,
                                               was_resumed, provider)
             except SoftTimeLimitExceeded:
                 if was_resumed:
@@ -587,7 +591,7 @@ class LaunchStackTask(HastexoTask):
                 if conn:
                     conn.close()
 
-    def check_stack(self, stack_outputs, was_resumed, provider):
+    def check_stack(self, provider_stack, was_resumed, provider):
         """
         Fetch stack outputs, check that the stack has a public IP address, a
         private key, and is network accessible after rebooting any servers.
@@ -602,9 +606,17 @@ class LaunchStackTask(HastexoTask):
         logger.debug("Verifying stack [%s] " % self.stack_name)
 
         # Get stack outputs
-        stack_ip = stack_outputs.get("public_ip")
-        stack_key = stack_outputs.get("private_key")
-        stack_password = stack_outputs.get("password")
+        stack_ip = provider_stack["outputs"].get("public_ip")
+        stack_password = provider_stack["outputs"].get("password")
+
+        # Get stack key; if no key was generated separately,
+        # fetch the key from stack outputs. If stack was resumed,
+        # use the key from the database.
+        if was_resumed:
+            stack_key = self.stack_key
+        else:
+            stack_key = provider_stack.get("private_key") or \
+                provider_stack["outputs"].get("private_key")
 
         if stack_ip is None or not stack_key:
             if was_resumed:
