@@ -495,7 +495,8 @@ class TestHastexoXBlock(TestCase):
             get_xblock_settings(),
             {"stack_id": stack.id,
              "reset": False,
-             "learner_id": stack.learner.id}
+             "learner_id": stack.learner.id,
+             "stack_key_type": ""}
         )
         self.assertEqual(response["status"], "LAUNCH_PENDING")
 
@@ -1637,13 +1638,55 @@ class TestHastexoXBlock(TestCase):
     def test_get_text_js_return_path(self):
         with patch('django.utils.translation.get_language',
                    return_value='es-419'):
-            text_js = self.block._get_text_js_url()
-            self.assertEqual(
+            text_js = self.block.get_js_urls()['text_js_url']
+            self.assertIn(
                 'public/js/translations/es-419/text.js', text_js)
 
     def test_get_text_js_return_none(self):
         with patch('django.utils.translation.get_language',
                    return_value='ar'):
             # ar is not in our SUPPORTED_LANGUAGES list
-            text_js = self.block._get_text_js_url()
-            self.assertIsNone(text_js)
+            js_urls = self.block.get_js_urls()
+            self.assertNotIn('text_js_url', js_urls)
+
+    def test_launch_new_window(self):
+        self.init_block()
+
+        with patch('webob.Request') as request:
+            request.cookies = Mock()
+            request.cookies = {
+                'sessionid': 'fake_sessionid',
+                'csrftoken': 'fake_csrf_token'}
+            response = self.block.launch_new_window(request)
+
+            self.assertEqual("200 OK", response.status)
+
+    def test_launch_new_window_unauthorized(self):
+        self.init_block()
+
+        with patch('webob.Request') as request:
+            request.cookies = Mock()
+            request.cookies = {'fake': 'cookie'}
+            response = self.block.launch_new_window(request)
+
+            self.assertEqual("401 Unauthorized", response.status)
+            self.assertEqual(b'Unauthorized', response.body)
+
+    def test_fullscreen_setting(self):
+        self.init_block()
+        settings = get_xblock_settings()
+        enable_fullscreen = self.block.get_enable_fullscreen(settings)
+
+        # by default the setting is to be inherited from global settings
+        self.assertEqual("inherit", self.block.enable_fullscreen)
+        self.assertEqual(settings.get("enable_fullscreen"), enable_fullscreen)
+
+        # If the XBlock attribute is used to enable/disable the feature
+        # the attribute value is used
+        self.block.enable_fullscreen = "true"
+        enable_fullscreen = self.block.get_enable_fullscreen(settings)
+        self.assertEqual(True, enable_fullscreen)
+
+        self.block.enable_fullscreen = "false"
+        enable_fullscreen = self.block.get_enable_fullscreen(settings)
+        self.assertEqual(False, enable_fullscreen)
