@@ -118,6 +118,7 @@ class HastexoTestCase(TestCase):
             port=self.port,
             run=self.stack_run,
             user=self.stack_user_name,
+            key=self.stack_key,
             providers=self.providers,
             hook_script=self.hook_script,
             hook_events=self.hook_events,
@@ -129,7 +130,8 @@ class HastexoTestCase(TestCase):
         self.kwargs = {
             "stack_id": stack.id,
             "reset": False,
-            "learner_id": self.learner.id
+            "learner_id": self.learner.id,
+            "stack_key_type": ""
         }
 
         # Patchers
@@ -209,7 +211,8 @@ class TestLaunchStackTask(HastexoTestCase):
         self.assertEqual(stack.error_msg, u"")
         provider.create_stack.assert_called_with(
             self.stack_name,
-            self.stack_run
+            self.stack_run,
+            key_type=""
         )
         self.mocks["ssh_to"].assert_called_with(
             self.stack_user_name,
@@ -275,7 +278,8 @@ class TestLaunchStackTask(HastexoTestCase):
         self.assertEqual(stack.error_msg, u"")
         provider.create_stack.assert_called_with(
             self.stack_name,
-            self.stack_run
+            self.stack_run,
+            key_type=""
         )
         self.mocks["ssh_to"].assert_called_with(
             self.stack_user_name,
@@ -540,6 +544,7 @@ class TestLaunchStackTask(HastexoTestCase):
         # Assertions
         self.assertEqual(stack.status, "RESUME_FAILED")
         self.assertEqual(stack.provider, self.providers[1]["name"])
+        self.assertEqual(self.stack_key, stack.key)
         provider.suspend_stack.assert_called()
 
     def test_undefined_capacity(self):
@@ -585,7 +590,8 @@ class TestLaunchStackTask(HastexoTestCase):
         self.assertEqual(stack.error_msg, u"")
         provider.create_stack.assert_called_with(
             self.stack_name,
-            self.stack_run
+            self.stack_run,
+            key_type=""
         )
 
     def test_use_next_provider_if_first_is_disabled(self):
@@ -614,7 +620,8 @@ class TestLaunchStackTask(HastexoTestCase):
         self.assertEqual(stack.error_msg, u"")
         provider2.create_stack.assert_called_with(
             self.stack_name,
-            self.stack_run
+            self.stack_run,
+            key_type=""
         )
 
     def test_use_next_provider_if_first_is_full(self):
@@ -652,7 +659,8 @@ class TestLaunchStackTask(HastexoTestCase):
         self.assertEqual(stack.error_msg, u"")
         provider2.create_stack.assert_called_with(
             self.stack_name,
-            self.stack_run
+            self.stack_run,
+            key_type=""
         )
 
     def test_all_providers_full(self):
@@ -714,11 +722,13 @@ class TestLaunchStackTask(HastexoTestCase):
         self.assertEqual(stack.error_msg, u"")
         provider1.create_stack.assert_called_with(
             self.stack_name,
-            self.stack_run
+            self.stack_run,
+            key_type=""
         )
         provider2.create_stack.assert_called_with(
             self.stack_name,
-            self.stack_run
+            self.stack_run,
+            key_type=""
         )
 
     def test_dont_use_next_provider_if_timeout(self):
@@ -781,7 +791,8 @@ class TestLaunchStackTask(HastexoTestCase):
         for m in self.mock_providers:
             m.create_stack.assert_called_with(
                 self.stack_name,
-                self.stack_run
+                self.stack_run,
+                key_type="",
             )
 
     def test_only_one_provider_configured(self):
@@ -932,6 +943,38 @@ class TestLaunchStackTask(HastexoTestCase):
             params="resume"
         )
 
+    def test_resume_key_from_db(self):
+        # Setup
+        provider = self.mock_providers[1]
+        provider.get_stack.side_effect = [
+            self.stacks["SUSPEND_COMPLETE"]
+        ]
+
+        resume_stack = copy.deepcopy(self.stacks["RESUME_COMPLETE"])
+        resume_stack["outputs"]["private_key"] = None
+        provider.resume_stack.side_effect = [
+            resume_stack
+        ]
+        self.update_stack({
+            "provider": self.providers[1]["name"],
+            "status": "SUSPEND_COMPLETE"
+        })
+
+        # Run
+        LaunchStackTask.run(**self.kwargs)
+
+        # Fetch stack
+        stack = self.get_stack()
+
+        # Assertions
+        self.assertEqual(stack.status, "RESUME_COMPLETE")
+        self.assertEqual(stack.provider, self.providers[1]["name"])
+        self.mocks["remote_exec"].assert_called_with(
+            self.mocks["ssh_to"].return_value,
+            self.read_from_contentstore,
+            params="resume"
+        )
+
     def test_resume_suspended_stack_transient_operational_error(self):
         """
         Try to resume a previously suspended stack, but simulate a
@@ -1016,6 +1059,10 @@ class TestLaunchStackTask(HastexoTestCase):
         # provider should still be unchanged.
         self.assertEqual(stack.provider, self.providers[1]["name"])
 
+        # When resume fails, we should still have the private key
+        # of the stack.
+        self.assertEqual(self.stack_key, stack.key)
+
     def test_resumed_stack_has_no_ip(self):
         # Setup
         provider = self.mock_providers[1]
@@ -1041,6 +1088,7 @@ class TestLaunchStackTask(HastexoTestCase):
         # Assertions
         self.assertEqual(stack.status, "RESUME_FAILED")
         self.assertEqual(stack.provider, self.providers[1]["name"])
+        self.assertEqual(self.stack_key, stack.key)
         provider.suspend_stack.assert_called()
 
     def test_timeout_resuming_stack(self):
@@ -1066,6 +1114,7 @@ class TestLaunchStackTask(HastexoTestCase):
         # Assertions
         self.assertEqual(stack.status, "LAUNCH_TIMEOUT")
         self.assertEqual(stack.provider, self.providers[1]["name"])
+        self.assertEqual(self.stack_key, stack.key)
 
     def test_resume_hook_empty(self):
         # Setup
@@ -1238,6 +1287,7 @@ class TestLaunchStackTask(HastexoTestCase):
 
         # Assertions
         self.assertEqual(stack.status, "CREATE_FAILED")
+        self.assertEqual('', stack.key)
         provider.delete_stack.assert_called_with(
             self.stack_name, False
         )
@@ -1262,6 +1312,7 @@ class TestLaunchStackTask(HastexoTestCase):
 
         # Assertions
         self.assertEqual(stack.status, "CREATE_FAILED")
+        self.assertEqual('', stack.key)
         provider.delete_stack.assert_called_with(
             self.stack_name, False
         )
